#####--------------------------------------------------------------------------*
#####--------------------------------------------------------------------------*
## Daniel Wollschläger <contact [at] dwoll.de>
## Befehle zum Buch "R kompakt" 3. Auflage (Springer)
## http://www.dwoll.de/r/
## Stand: 04.01.21
##
## Vgl. Abschnitt 1.1.4 und 2.2, um die folgenden Befehle auszuführen.
##
## Befehle, die eine Benutzerinteraktion notwendig machen, oder
## nicht erwünschte Nebenwirkungen haben, sind auskommentiert.
##
## Um alle Teile des Skripts ausführen zu können, müssen die Pakete
## car, DescTools, dplyr, forcats, ggbeeswarm, ggplot2, modeest, pROC, psych,
## rgl, RSQLite, tidyr
## samt Abhängigkeiten installiert sein, vgl. Abschnitt 1.2.8.
pWants <- c("car", "DescTools", "dplyr", "forcats", "ggbeeswarm", "ggplot2",
            "pROC", "psych", "rgl", "RSQLite", "tidyr")
pHas   <- pWants %in% rownames(installed.packages())
if(any(!pHas)) install.packages(pWants[!pHas], repos="https://cran.rstudio.com/")
#####--------------------------------------------------------------------------*
#####--------------------------------------------------------------------------*

####*--------------------------------------------------------------------------*
##### 1 Erste Schritte
#####--------------------------------------------------------------------------*

####---------------------------------------------------------------------------*
##### 1.1 Vorstellung ----
subW <- readRDS(url("http://dwoll.de/r/rKompakt/dat_subW.rds"))
head(subW)

library(ggplot2)
library(dplyr)
ggplot(subW, aes(x=group, y=iq)) +
    geom_boxplot(outlier.shape=NA, size=0.7) +
    geom_point(position=position_jitter(w=0.05), size=2) +
    ggtitle("IQ-Verteilung pro Gruppe") +
    theme_classic()

aggregate(iq ~ group, FUN=mean, data=subW)
subW |>
    group_by(sex, group) |>
    summarise(iq.M=mean(iq),
              iq.SD=sd(iq)) |>
    ungroup()

with(subW, cor(cbind(DV.pre, DV.post, DV.fup)))
anova(lm(DV.post ~ group, data=subW))
fit <- lm(DV.fup ~ DV.pre, data=subW)
summary(fit)
ggplot(subW, aes(x=DV.pre, y=DV.fup)) +
    geom_point() +
    geom_smooth(method=lm) +
    ggtitle("Regression f'up auf prä") +
    theme_classic()

##### 1.1.1 ----
##### 1.1.2 ----
1 + 1

##### 1.1.3 ----
##### 1.1.4 ----
##### 1.1.5 ----

#####--------------------------------------------------------------------------*
##### 1.2 Grundlegende Elemente ----
##### 1.2.1 ----
1 + 1

2 * (4
     -5)
## q()

##### 1.2.2 ----
getOption("width")
## op <- option(width=70)
## options(op)

##### 1.2.3 ----
history()
ls()

##### 1.2.4 ----
12^2 + 1.5*10
sin(pi/2) + sqrt(abs(-4))

##### 1.2.5 ----
## q()
round(1.271)
round(pi, digits=2)

##### 1.2.6 ----
## help.start()
## ?"/"
## help()

##### 1.2.7 ----
.libPaths()
installed.packages()
library()
sessionInfo()

#####--------------------------------------------------------------------------*
##### 1.3 Datenstrukturen: Klassen, Objekte, Datentypen ----
##### 1.3.1 ----
##### 1.3.2 ----
x1 <-  4.5
x1  =  4.5
x1 * 2
x2 <- 10
x1^x1 - x2

##### 1.3.3 ----
print(x1)
x1
(x1 <- 4.5)

##### 1.3.4 ----
ls()
## rm(list=ls(all.names=TRUE))
age <- 22
rm(age)
## age

##### 1.3.5 ----
NULL
TRUE
FALSE
T
F
3.14
"Hello"
charVar <- "asdf"
mode(charVar)
as.character(1.23)
as.logical(2)

##### 1.3.6 ----
TRUE == TRUE
TRUE == FALSE
!TRUE
!FALSE
TRUE != TRUE
TRUE != FALSE
isTRUE(TRUE)
isTRUE(FALSE)
 TRUE & TRUE
 TRUE & FALSE
FALSE & FALSE
FALSE & TRUE
 TRUE | TRUE
 TRUE | FALSE
FALSE | FALSE
FALSE | TRUE
4 <  8
7 <  3
4 >  4
4 >= 4

0.1 + 0.2 == 0.3
1 %/% 0.1

all.equal(0.123450001, 0.123450000)
0.123400001 == 0.123400000
all.equal(0.12345001, 0.12345000)
isTRUE(all.equal(0.12345001,  0.12345000))

####*--------------------------------------------------------------------------*
##### 2 Daten importieren und exportieren ----
#####--------------------------------------------------------------------------*

##### 2.1 Daten im Textformat lesen und schreiben ----
myDf <- data.frame(IV=factor(rep(c("A", "B"), 5)), DV=rnorm(10))
write.table(myDf, file="data.txt")
myDf <- read.table("data.txt", header=TRUE)
str(myDf)
head(myDf, n=4)

##### 2.2 R-Objekte laden und speichern ----
myDf <- data.frame(IV=factor(rep(c("A", "B"), 5)), DV=rnorm(10))
saveRDS(myDf, file="myDf.rds")
myDf <- readRDS("myDf.rds")
dump("myDf", file="dumpMyDf.txt")
source("dumpMyDf.txt")

#####--------------------------------------------------------------------------*
##### 2.3 Daten im Editor bearbeiten ----
myDf <- data.frame(IV=factor(rep(c("A", "B"), 5)), DV=rnorm(10))
## fix(myDf)

##### 2.4 Daten mit anderen Programmen austauschen ----
##### 2.4.1 ----
## myDf <- read.table("clipboard", sep="\t", dec=",")

IQ     <- rnorm(2*100, mean=100, sd=15)
rating <- sample(LETTERS[1:3], 2*50, replace=TRUE)
sex    <- factor(rep(c("f", "m"), times=50))
myDf   <- data.frame(sex, IQ, rating, stringsAsFactors=FALSE)
## für Daten des Buches diese Datei laden:
# load("data_db.Rdata")

library(RSQLite)
drv <- dbDriver("SQLite")
con <- dbConnect(drv, dbname=":memory:")# "myDf.db")
dbWriteTable(con, name="MyDataFrame", value=myDf, row.names=FALSE)
dbListTables(con)
dbListFields(con, "MyDataFrame")
out <- dbReadTable(con, "MyDataFrame")
head(out, n=4)
dbGetQuery(con, "SELECT sex, AVG(IQ) AS mIQ, SUM(IQ) AS sIQ FROM MyDataFrame GROUP BY sex")
dbRemoveTable(con, "MyDataFrame")
dbDisconnect(con)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:RSQLite))

####*--------------------------------------------------------------------------*
##### 3 Elementare Datenverarbeitung ----
#####--------------------------------------------------------------------------*

#####--------------------------------------------------------------------------*
##### 3.1 Vektoren ----
##### 3.1.1 ----
numeric(4)
(age    <- c(18, 20, 30, 24, 23, 21))
addAge  <- c(27, 21, 19)
(ageNew <- c(age, addAge))
length(age)
(chars <- c("lorem", "ipsum", "dolor"))
LETTERS
letters

##### 3.1.2 ----
age[4]
(ageLast <- age[length(age)])
age[length(age) + 1]

idx <- c(1, 2, 4)
age[idx]
age[c(3, 5, 6)]
age[c(1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6)]

age[-3]
age[-idx]

age[4] <- 27
age
age[idx] <- c(17, 30, 25)
age

charVec1 <- c("Z", "Y", "X")
charVec1[c(4, 5, 6)] <- c("W", "V", "U")
charVec1
(charVec2 <- c(charVec1, "T", "S", "R"))

vec <- c(10, 20, 30, 40, 50)
vec <- vec[c(-4, -5)]
vec

##### 3.1.3 ----
charVec4 <- "word"
numVec   <- c(10, 20, 30)
(combVec <- c(charVec4, numVec))
mode(combVec)

##### 3.1.4 ----
(namedVec1 <- c(elem1="first", elem2="second"))
namedVec1["elem1"]
(namedVec2 <- c(val1=10, val2=-12, val3=33))
names(namedVec2)
names(namedVec2) <- c("A", "B", "C")
namedVec2

#####--------------------------------------------------------------------------*
##### 3.2 Logische Operatoren ----
##### 3.2.1 ----
age <- c(17, 30, 30, 24, 23, 21)
age < 24
x <- c(2, 4, 8)
y <- c(3, 4, 5)
x == y
x < y
(age <= 20) | (age >= 30)
(age > 20)  & (age < 30)

x <- c(4, 5, 6)
y <- c(4, 5, 6)
z <- c(1, 2, 3)
all.equal(x, y)
all.equal(y, z)
isTRUE(all.equal(y, z))

res <- age < 24
sum(res)

##### 3.2.2 ----
age[c(TRUE, FALSE, TRUE, TRUE, FALSE, TRUE)]
(idx <- (age <= 20) | (age >= 30))
age[idx]
age[(age >= 30) | (age <= 20)]

vecNA   <- c(-3, 2, 0, NA, -7, 5)
(logIdx <- vecNA > 0)
vecNA[logIdx]
na.omit(vecNA[logIdx])

(numIdx <- which(logIdx))
vecNA[numIdx]

#####--------------------------------------------------------------------------*
##### 3.3 Systematische und zufällige Wertefolgen erzeugen ----
##### 3.3.1 ----
20:26
26:20
-4:2
-(4:2)
seq(from=2, to=12, by=2)
seq(from=2, to=11, by=2)
seq(from=0, to=-1, length.out=5)

age <- c(18, 20, 30, 24, 23, 21)
seq_along(age)

seq_len(length(age))

##### 3.3.2 ----
rep(1:3, times=5)
rep(c("A", "B", "C"), times=c(2, 3, 4))
rep(age,  each=2)

##### 3.3.3 ----
sample(1:6, size=20, replace=TRUE)
sample(c("rot", "grün", "blau"), size=8, replace=TRUE)

##### 3.3.4 ----
runif(5, min=1, max=6)
rbinom(20, size=5, prob=0.3)
rnorm(6, mean=100, sd=15)

#####--------------------------------------------------------------------------*
##### 3.4 Daten transformieren ----
##### 3.4.1 ----
vec <- c(10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20)
rev(vec)
vec <- c(10, 12, 1, 12, 7, 16, 6, 19, 10, 19)
sort(vec)
(idxDec <- order(vec, decreasing=TRUE))
vec[idxDec]
sort(c("D", "E", "10", "A", "F", "E", "D", "4", "E", "A"))

##### 3.4.2 ----
myColors  <- c("red", "green", "blue", "yellow", "black")
(randCols <- sample(myColors, length(myColors), replace=FALSE))

P       <- 3
Nj      <- c(4, 3, 5)
IV      <- rep(1:P, Nj)
(IVrand <- sample(IV, length(IV), replace=FALSE))

##### 3.4.3 ----
vec <- rep(c("rot", "grün", "blau"), 10)
sample(vec, 5, replace=FALSE)

selIdx1 <- seq(1, length(vec), by=10)
vec[selIdx1]

selIdx2 <- rbinom(length(vec), size=1, prob=0.1) == 1
vec[selIdx2]

unique(c(1, 1, 1, 3, 3, 4, 4))
length(unique(c("A", "B", "C", "C", "B", "B", "A", "C", "C", "A")))

is.element(c(29, 23, 30, 17, 30, 10), c(30, 23))
x <- c("A", "Z", "B")
x[x %in% c("A", "B", "C", "D", "E")]
x[!(x %in% c("A", "B", "C", "D", "E"))]

##### 3.4.4 ----
age <- c(18, 20, 30, 24, 23, 21)
age/10
(age/2) + 5
vec1 <- c(3, 4, 5, 6)
vec2 <- c(-2, 2, -2, 2)
vec1*vec2

##### 3.4.4.1 ----
vec1 <- c(2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24)
vec2 <- c(2, 4, 6, 8, 10)
c(length(age), length(vec1), length(vec2))
vec1*age
vec2*age

##### 3.4.4.2 ----
(zAge <- scale(age))
as.vector(zAge)

newSd   <- 15
newMean <- 100
(newAge <- (as.vector(zAge)*newSd) + newMean)

##### 3.4.4.3 ----
rank(c(3, 1, 2, 3))

##### 3.4.5 ----
height <- c(1.78, 1.91, 1.89, 1.83, 1.64)
weight <- c(65, 89, 91, 75, 73)
(bmi   <- weight / (height^2))

quest1  <- c(FALSE, FALSE, FALSE, TRUE,  FALSE, TRUE, FALSE, TRUE)
quest2  <- c(TRUE,  FALSE, FALSE, FALSE, TRUE,  TRUE, TRUE,  FALSE)
quest3  <- c(TRUE,  TRUE,  TRUE,  TRUE,  FALSE, TRUE, FALSE, FALSE)
(sumVar <- quest1 + quest2 + quest3)

##### 3.4.6 ----
myColors <- c("red", "purple", "blue", "blue", "red")
farben   <- character(length(myColors))
farben[myColors == "red"]    <- "rot"
farben[myColors == "purple"] <- "violett"
farben[myColors == "blue"]   <- "blau"
farben

library(dplyr)                                    # für recode()
recode(myColors, red="rot", blue="blau", purple="violett")
recode(myColors, red="basic", blue="basic", .default="complex")

orgVec <- c(5, 9, 11, 8, 9, 3, 1, 13, 9, 12, 5, 12, 6, 3, 17, 5, 8, 7)
cutoff <- 10
(reVec <- ifelse(orgVec <= cutoff, orgVec, cutoff))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:dplyr))

##### 3.4.7 ----
IQ    <- c(112, 103, 87, 86, 90, 101, 90, 89, 122, 103)
IQcls <- numeric(length(IQ))
IQcls[IQ <= 100]                <- 1
IQcls[(IQ > 100) & (IQ <= 115)] <- 2
IQcls[IQ > 115]                 <- 3
IQcls

library(dplyr)
case_when((IQ <= 100)              ~ 1,
          (IQ > 100) & (IQ <= 115) ~ 2,
          (IQ > 115)               ~ 3)

ifelse(IQ >= 100, "hi", "lo")

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:dplyr))

#####--------------------------------------------------------------------------*
##### 3.6 Gruppierungsfaktoren ----
##### 3.6.1 ----
sex     <- c("m", "f", "f", "m", "m", "m", "f", "f")
(sexFac <- factor(sex))
factor(c(1, 1, 3, 3, 4, 4), levels=1:5)
(sexNum <- rbinom(10, size=1, prob=0.5))
factor(sexNum, levels=0:1, labels=c("man", "woman"))
levels(sexFac) <- c("female", "male")
sexFac

nlevels(sexFac)
summary(sexFac)
levels(sexFac)
str(sexFac)
unclass(sexFac)
as.character(sexFac)

##### 3.6.2 ----
(status   <- factor(c("hi", "lo", "hi")))
status[4] <- "mid"
status
levels(status) <- c(levels(status), "mid")
status[4] <- "mid"
status

hiNotHi <- status
levels(hiNotHi) <- list(hi="hi", notHi=c("mid", "lo"))
hiNotHi

library(forcats)
fct_collapse(status, notHi=c("mid", "lo"))
status[1:2]
(newStatus <- droplevels(status[1:2]))
fct_drop(status[1:2], "mid")

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:forcats))

##### 3.6.3 ----
(ordStat <- ordered(status, levels=c("lo", "mid", "hi")))
ordStat[1] > ordStat[2]

##### 3.6.4 ----
vec <- c(3, 4, 3, 2, 1, 4, 1, 1)
factor(vec)
factor(vec, levels=c(4, 3, 2, 1))
factor(vec, levels=1:4, labels=c("one", "two", "three", "four"))

##### 3.6.5 ----
(fac1 <- factor(rep(c("A", "B"), c(5, 5))))
(fac2 <- gl(2, 5, labels=c("less", "more"), ordered=TRUE))
expand.grid(IV1=gl(2, 2, labels=c("a", "b")), IV2=gl(2, 1))

##### 3.6.6 ----
IQ    <- rnorm(100, mean=100, sd=15)
IQfac <- cut(IQ, breaks=c(0, 85, 115, Inf), labels=c("lo", "mid", "hi"))
IQfac[1:5]

medSplit <- cut(IQ, breaks=c(-Inf, median(IQ), Inf))
summary(medSplit)

#####--------------------------------------------------------------------------*
##### 3.7 Deskriptive Kennwerte numerischer Daten ----
age <- c(17, 30, 30, 25, 23, 21)
summary(age)

##### 3.7.1 ----
sum(age)
cumsum(age)

diff(age)

prod(age)
cumprod(age)
factorial(5)

##### 3.7.2 ----
max(age)
range(c(17, 30, 30, 25, 23, 21))

which.min(age)
vec <- c(-5, -8, -2, 10, 9)
val <- 0
which.min(abs(vec-val))

diff(range(c(17, 30, 30, 25, 23, 21)))

##### 3.7.3 ----
mean(1, 3, 7)
mean(c(1, 3, 7))
age <- c(17, 30, 30, 25, 23, 21)
mean(age)

sort(age)
median(age)

vec <- c(11, 22, 22, 33, 33, 33, 33)
library(DescTools)
Mode(vec)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))

##### 3.7.4 ----
age <- c(17, 30, 30, 25, 23, 21)
sum(age <= 25)
100 * (sum(age <= 25) / length(age))
library(DescTools)
PercentRank(age)

(quant <- quantile(age))
quant[c("25%", "50%")]

vec <- sample(seq(0, 1, by=0.01), 1000, replace=TRUE)
quantile(vec, probs=c(0, 0.2, 0.4, 0.6, 0.8, 1))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))

##### 3.7.5 ----
age <- c(17, 30, 30, 25, 23, 21)
N   <- length(age)
var(age)

((N-1) / N) * var(age)

sd(age)
sqrt((N-1) / N) * sd(age)

##### 3.7.6 ----
x <- c(17, 30, 30, 25, 23, 21)
y <- c(1, 12, 8, 10, 5, 3)
cov(x, y)

N <- length(x)
((N-1) / N) * cov(x, y)

cor(x, y)

##### 3.7.7 ----
age <- c(17, 30, 30, 25, 23, 21)
sd(age)
quantile(age)
IQR(age)
mean(abs(age-mean(age)))
mean(abs(age-median(age)))
mad(age)

library(psych)
ageWins <- winsor(age, trim=0.2)
sd(ageWins)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:psych))

##### 3.7.8 ----
Njk   <- 2
P     <- 2
Q     <- 3
sex   <- factor(rep(c("f", "m"),       times=Q*Njk))
group <- factor(rep(c("T", "WL", "CG"), each=P*Njk))
IQ    <- round(rnorm(Njk*P*Q, mean=100, sd=15))
(tapRes <- tapply(IQ, group, FUN=mean))
tapply(IQ, list(sex, group), FUN=mean)
tapRes[group]

#####--------------------------------------------------------------------------*
##### 3.8 Matrizen ----
age <- c(17, 30, 30, 25, 23, 21)
matrix(age, nrow=3, ncol=2, byrow=FALSE)
(ageMat <- matrix(age, nrow=2, ncol=3, byrow=TRUE))

##### 3.8.1 ----
age    <- c(17, 30, 30, 25, 23, 21)
ageMat <- matrix(age, nrow=2, ncol=3, byrow=FALSE)
dim(ageMat)
nrow(ageMat)
ncol(ageMat)

t(ageMat)
c(ageMat)

##### 3.8.2 ----
ageMat[1, 2]
ageMat[2, 2] <- 24
ageMat[2, 2]
ageMat[2,  ]
ageMat[ , 1]
ageMat[ , 1, drop=FALSE]
ageMat[ , 2:3]

idxVec <- c(1, 3, 4)
ageMat[idxVec]

(idxMatLog <- ageMat >= 25)
ageMat[idxMatLog]

##### 3.8.3 ----
age     <- c(19, 19, 31, 19, 24)
weight  <- c(95, 76, 94, 76, 76)
height  <- c(197, 178, 189, 184, 173)
rbind(age, weight, height)
(mat <- cbind(age, weight, height))

##### 3.8.4 ----
(rowOrder1 <- order(mat[ , "age"]))
mat[rowOrder1, ]

rowOrder2 <- order(mat[ , "age"], partial=mat[ , "weight"])
mat[rowOrder2, ]

rowOrder3 <- order(mat[ , "weight"], -mat[ , "height"])
mat[rowOrder3, ]

##### 3.8.5 ----
sum(mat)
rowSums(mat)
mean(mat)
colMeans(mat)

apply(mat, MARGIN=2, FUN=sum)
apply(mat, MARGIN=1, FUN=range)
apply(mat, MARGIN=2, FUN=mean, trim=0.1)

##### 3.8.6 ----
cov(mat)
cor(mat)
cov.wt(mat, method="ML")

vec <- rnorm(nrow(mat))
cor(mat, vec)

#####--------------------------------------------------------------------------*
##### 3.9 Häufigkeitsauszählungen ----
##### 3.9.1 ----
(myLetters <- sample(LETTERS[1:5], 12, replace=TRUE))
## für Daten des Buches diese Datei laden:
# load("data_myLetters.Rdata")
(tab <- xtabs(~ myLetters))
names(tab)
tab["B"]

(relFreq <- proportions(tab))
cumsum(relFreq)

letFac <- factor(myLetters, levels=c(LETTERS[1:5], "Q"))
letFac
xtabs(~ letFac)

##### 3.9.2 ----
N       <- 10
(work   <- factor(sample(c("home", "office"), N, replace=TRUE)))
(sex    <- factor(sample(c("f", "m"),         N, replace=TRUE)))
(counts <- sample(0:5, N, replace=TRUE))
## für Daten des Buches diese Datei laden:
# load("data_xtabs.Rdata")
(absFreq <- xtabs(~ sex + work))
xtabs(counts ~ sex + work)
(relFreq <- proportions(absFreq))
proportions(absFreq, 2)

(group <- factor(sample(c("A", "B"), 10, replace=TRUE)))
## für Daten des Buches diese Datei laden:
# load("data_xtabs.Rdata")
ftable(work, sex, group, row.vars="work", col.vars=c("sex", "group"))

persons <- data.frame(sex, work, counts)

##### 3.9.3 ----
## für Daten des Buches diese Datei laden:
# load("data_xtabs.Rdata")
apply(     xtabs(~ sex + work, data=persons), MARGIN=1, FUN=sum)
colMeans(  xtabs(~ sex + work, data=persons))
addmargins(xtabs(~ sex + work, data=persons), c(1, 2), FUN=mean)

#####--------------------------------------------------------------------------*
##### 3.10 Codierung, Identifikation und Behandlung fehlender Werte ----
##### 3.10.1 ----
(vec1 <- c(10, 20, NA, 40, 50, NA))
length(vec1)
is.na(vec1)

vec2   <- c(NA, 7, 9, 10, 1, 8)
(matNA <- rbind(vec1, vec2))
is.na(matNA)

any(is.na(vec1))
sum(is.na(vec1))
which(is.na(vec1))

##### 3.10.2 ----
vec <- c(30, 25, 23, 21, -999, 999)
is.na(vec) <- vec %in% c(-999, 999)
vec

(mat <- matrix(c(30, 25, 23, 21, -999, 999), nrow=2, ncol=3))
is.na(mat) <- mat %in% c(-999, 999)
mat

##### 3.10.3 ----
na.omit(vec)
sd(na.omit(vec))

sum(vec)
sum(vec, na.rm=TRUE)
apply(matNA, MARGIN=1, FUN=mean, na.rm=TRUE)

##### 3.10.4 ----
##### 3.10.4.1 ----
ageNA  <- c(18, NA, 27, 22)
DV1    <- c(NA, 1, 5, -3)
DV2    <- c(9, 4, 2, 7)
(matNA <- cbind(ageNA, DV1, DV2))

na.omit(matNA)
colMeans(na.omit(matNA))

cov(matNA, use="complete.obs")

##### 3.10.4.2 ----
rowMeans(matNA)
rowMeans(matNA, na.rm=TRUE)
cov(matNA, use="pairwise.complete.obs")

##### 3.10.5 ----
##### 3.10.6 ----

#####--------------------------------------------------------------------------*
##### 3.11 Zeichenketten verarbeiten ----
##### 3.11.1 ----
nchar("ABCDEF")
nchar(c("A", "BC", "DEF"))

paste("group", LETTERS[1:5], sep="_")
paste(1:5, letters[1:5], sep=".", collapse=" ")

##### 3.11.2 ----
substring(c("ABCDEF", "GHIJK", "LMNO", "PQR"), first=4, last=5)
strsplit(c("abc_def_ghi", "jkl_mno"), split="_")
strsplit("Xylophon", split=NULL)

##### 3.11.3 ----
grep( "A[BC][[:blank:]]", c("AB ", "AB", "AC ", "A "))
grepl("A[BC][[:blank:]]", c("AB ", "AB", "AC ", "A "))
pat    <- "[[:upper:]]+"
txt    <- c("abcDEFG", "ABCdefg", "abcdefg")
(start <- regexpr(pat, txt))
len    <- attr(start, "match.length")
end    <- start + len - 1
substring(txt, start, end)

glob2rx("asdf*.txt")

##### 3.11.4 ----
charVec <- c("ABCDEF", "GHIJK", "LMNO", "PQR")
substring(charVec, 4, 5) <- c("..", "xx", "++", "**")
charVec

 sub("em", "XX", "Lorem ipsum dolor sit Lorem ipsum")
gsub("em", "XX", "Lorem ipsum dolor sit Lorem ipsum")

#####--------------------------------------------------------------------------*
##### 3.12 Datum und Uhrzeit ----
##### 3.12.1 ----
Sys.Date()

(myDate <- as.Date("01.11.1974", format="%d.%m.%Y"))
format(myDate, format="%d.%m.%Y")

##### 3.12.2 ----
Sys.time()
date()

(myTime   <- as.POSIXct("2009-02-07 09:23:02"))
charDates <- c("05.08.1972, 03:37", "31.03.1981, 12:44")
(lDates   <- strptime(charDates, format="%d.%m.%Y, %H:%M"))
lDates$mday

##### 3.12.3 ----
(myDate <- as.Date("01.11.1974", format="%d.%m.%Y"))
myDate + 365

(diffDate <- as.Date("1976-06-19") - myDate)
as.numeric(diffDate)
myDate + diffDate

round(lDates, units="days")
trunc(lDates, units="years")

seq(ISOdate(2010, 5, 1), ISOdate(2015, 5, 1), by="years")
seq(ISOdate(1997, 10, 22), by="2 weeks", length.out=4)

secsPerDay <- 60 * 60 * 24
randDates  <- ISOdate(1995, 6, 13) + sample(0:(28*secsPerDay), 100, replace=TRUE)
randWeeks  <- cut(randDates, breaks="week")
summary(randWeeks)

####*--------------------------------------------------------------------------*
##### 4 Datensätze aufbereiten und aggregieren mit dplyr ----
#####--------------------------------------------------------------------------*
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
head(myDf1)

fac   <- factor(c("CG", "T1", "T2"))
DV1   <- c(14, 22, 18)
DV2   <- c("red", "blue", "blue")
myDf2 <- data.frame(fac, DV1, DV2)
str(myDf2)

##### 4.1 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

df_aggr <- myDf1 |>
    rename(class=group) |>
    mutate(IQ_Z=scale(IQ),
           IQ_ZSq=IQ_Z^2) |>
    group_by(class) |>
    summarise(M_IQ_ZSq=mean(IQ_ZSq))

df_aggr

myDf2 <- rename(myDf1, class=group)
myDf3 <- mutate(myDf2, IQ_Z=scale(IQ), IQ_Zsq=IQ_Z^2)
myDf4 <- group_by(myDf3, class)
summarise(myDf4, M_IQ_ZSq=mean(IQ_Zsq))

myDf1 |>
    function(d) { lm(IQ ~ age + rating, data=d) }

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.2 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

myDf1 |>
    rename(score=rating,
             fac=group)

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.3 ----
##### 4.3.1 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

myDf1 |>
    select(group, IQ)

myDf1 |>
    select(matches("^i.+"))

myDf1 |>
    select(where(is.numeric))

myDf1 |>
    select(!where(is.numeric) & starts_with("g"))

myDf1 |>
    select(-sex, -matches("a.+"))

myDf1 |>
    select(-(group:IQ))

##### 4.3.2 ----
myDf1 |>
    filter(group %in% c("CG", "WL"),
           IQ > 90)

myDf1 |>
    filter((IQ < 90) | (IQ > 110))

myDf1 |>
    slice(5:7)

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.4 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)
library(forcats)

myDf1 |>
    mutate(married=sample(c(TRUE, FALSE), n(), replace=TRUE))

myDf1 |>
    mutate(group=fct_collapse(group, CG_WL=c("CG", "WL")),
           ratingSq=rating^2,
           ratingSqZ=scale(ratingSq))

myDf1 |>
    mutate(age_even=if_else((age %% 2) == 0, "g", "u"),
           sex_IQ=case_when(
                   ((sex == "f") & (IQ <  100)) ~ "female_lo",
                   ((sex == "f") & (IQ >= 100)) ~ "female_hi",
                   ((sex == "m") & (IQ <  100)) ~ "male_lo",
                   ((sex == "m") & (IQ >= 100)) ~ "male_hi",
                   TRUE                         ~ "other"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))
try(detach(package:forcats))

##### 4.5 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

myDf1 |>
    select(sex, group) |>
    distinct()

myDf1 |>
    distinct(sex, group)

myDf1 |>
    n_distinct()

myDf9999           <- myDf1
myDf9999$IQ[2]     <- 9999
myDf9999$rating[3] <- 9999

(myDfNA <- myDf9999 |>
    mutate(IQ=na_if(IQ, 9999),
           rating=na_if(rating, 9999)))

myDfNA |>
    na.omit()

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.6 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

myDf1 |>
    arrange(rating)

myDf1 |>
    arrange(group, desc(IQ))

myDf1 |>
    select(group, age, everything())

myDf1 |>
    relocate(group, .after=rating)

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.7 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

myDf2 <- data.frame(id=21,
                    sex=factor("f", levels=c("f", "m")),
                    age=48,
                    rating=3)

bind_rows(myDf1, myDf2) |>
    tail(n=3)

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.8 ----
library(dplyr)

(IDDV <- data.frame(ID=factor(rep(1:3, each=2)),
                    DV=round(rnorm(6, 100, 15))))

(IV <- data.frame(ID=factor(1:3),
                  IV=factor(c("A", "B", "A")),
                  sex=factor(c("f", "f", "m"))))

IDDV |>
    left_join(IV, by="ID")

(dfA <- data.frame(ID=1:4,
                   initials=c("AB", "CD", "EF", "GH"),
                   IV1=c("-", "-", "+", "+"),
                   DV1=c(10, 10, 11, 14),
                   stringsAsFactors=FALSE))

(dfB <- data.frame(ID_mod=3:6,
                   initials=c("EF", "GH", "IJ", "KL"),
                   IV2=c("A", "B", "A", "B"),
                   DV2=c(92, 79, 101, 81),
                   stringsAsFactors=FALSE))

dfA |>
    left_join(dfB, by=c("ID"="ID_mod", "initials"))

dfA |>
    right_join(dfB, by=c("ID"="ID_mod", "initials"))

dfA |>
    full_join(dfB, by=c("ID"="ID_mod", "initials"))

dfA |>
    inner_join(dfB, by=c("ID"="ID_mod", "initials"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.9 ----
library(dplyr)
library(tidyr)

Nj    <- 2
P     <- 2
Q     <- 3
id    <- 1:(P*Nj)
DV_t1 <- round(rnorm(P*Nj, -1, 1), 2)
DV_t2 <- round(rnorm(P*Nj,  0, 1), 2)
DV_t3 <- round(rnorm(P*Nj,  1, 1), 2)
IVbtw <- factor(rep(c("A", "B"), Nj))
(datW <- data.frame(id, IVbtw, DV_t1, DV_t2, DV_t3))

datL <- datW |>
    pivot_longer(cols=starts_with("DV_"),
                 names_to="time", values_to="DV",
                 names_prefix="DV_")

datL

datL |>
    pivot_wider(id_cols=c(id, IVbtw),
                names_from=time, values_from=DV,
                names_prefix="DV_")

datL |>
    mutate(DVsq=DV^2) |>
    pivot_wider(id_cols=c(id, IVbtw),
                names_from=time, values_from=c(DV, DVsq))

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))
try(detach(package:tidyr))

##### 4.10 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

myDf1_grp <- myDf1 |>
    group_by(sex) |>
    mutate(IQ_rank=rank(IQ)) |>
    arrange(sex, IQ)

myDf1_grp

myDf1_grp |>
    ungroup()

myDf1_grp |>
    slice_head(n=2)

myDf1_grp |>
    head(n=2)

myDf1 |>
    group_by(group) |>
    summarise(age_M=mean(age),
              age_SD=sd(age),
              IQ_M=mean(IQ),
              IQ_SD=sd(IQ),
              n=n())

df_aggr <- myDf1 |>
    group_by(sex, group) |>
    summarise(rating_M=mean(rating))

df_aggr

df_aggr |>
    summarise(n=n())

myDf1 |>
    group_by(sex) |>
    summarise(n=n())

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

##### 4.11 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
library(dplyr)

myDf1 |>
    count(sex, group)

as.data.frame(xtabs(~ sex + group, data=myDf1))

myDf1 |>
    count(sex, group, .drop=FALSE)

myDf1 |>
    add_count(sex, group)

myDf1 |>
    count(sex, group) |>
    mutate(freq_rel=n / sum(n))

myDf1 |>
    add_count(sex, group) |>
    mutate(freq_rel=n / n())

myDf1 |>
    count(sex, group, name="n_sex_group") |>
    group_by(sex) |>
    mutate(n_sex=sum(n_sex_group),
           freq_cond_rel=n_sex_group / n_sex) |>
    ungroup()

myDf1 |>
    add_count(sex, name="n_sex") |>
    add_count(sex, group, name="n_sex_group") |>
    mutate(freq_cond_rel=n_sex_group / n_sex) |>
    select(id, sex, group, n_sex, n_sex_group, freq_cond_rel)

## entferne die (ggf. automatisch) geladenen Zusatzpakete
try(detach(package:dplyr))

####*--------------------------------------------------------------------------*
#### 5 R Dokumente und Notebooks erstellen ----
#####--------------------------------------------------------------------------*

####*--------------------------------------------------------------------------*
##### 6 Hilfsmittel für die Inferenzstatistik ----
#####--------------------------------------------------------------------------*

#####--------------------------------------------------------------------------*
##### 6.1 Formulierung linearer Modelle ----
Y ~ X1
Y ~ X1 + X2 - 1
Y ~ F1
Y ~ F1 + F2 + F1:F2
Y ~ X1 + F1
Y ~ X1*X2

log(Y) ~ abs(X)
Y ~ I(X*2)

#####--------------------------------------------------------------------------*
##### 6.2 Funktionen von Zufallsvariablen ----
##### 6.2.1 ----
dbinom(7, 10, 0.5)

##### 6.2.2 ----
pbinom(7, size=10, prob=0.5)
pnorm(c(-Inf, 0, Inf), mean=0, sd=1)
  pnorm(1.645, mean=0, sd=1, lower.tail=FALSE)
1-pnorm(1.645, mean=0, sd=1, lower.tail=TRUE)

##### 6.2.3 ----
qnorm(1-(0.05/2), 0, 1)
qt(1-0.01, 18, 0)

##### 6.3 ----
datL <- readRDS(url("http://dwoll.de/r/rKompakt/datL.rds"))
datW <- readRDS(url("http://dwoll.de/r/rKompakt/datW.rds"))

####*--------------------------------------------------------------------------*
##### 7 Korrelations- und Regressionsanalyse ----
#####--------------------------------------------------------------------------*

#####--------------------------------------------------------------------------*
##### 7.1 Test auf Korrelation ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
cor.test(~ attention + verbal, data=datW)

#####--------------------------------------------------------------------------*
##### 7.2 Einfache lineare Regression ----
##### 7.2.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(fitI <- lm(verbal ~ iq, data=datW))
lm(scale(verbal) ~ scale(iq), data=datW)

## ggplot2
library(ggplot2)
library(dplyr)

datW_M <- datW |>
    summarise(verbal=mean(verbal),
              iq=mean(iq))

ggplot(datW, aes(x=verbal, y=iq)) +
    geom_point() +
    geom_smooth(method=lm) +
    geom_point(data=datW_M, color="red", shape=4, size=5, stroke=2) +
    ggtitle("Daten und Vorhersage durch Regression") +
    xlab("Prädiktor") +
    ylab("Kriterium und Vorhersage") +
    theme_classic()

## base R
par(lend=2)
plot(verbal ~ iq, data=datW, pch=20,
     xlab="Prädiktor",
     ylab="Kriterium und Vorhersage",
     xaxs="i", main="Daten und Vorhersage durch Regression")
means <- aggregate(cbind(verbal, iq) ~ 1, FUN=mean, data=datW)
pred  <- predict(fitI, interval="confidence", level=0.95)
hOrd  <- with(datW, order(iq))
with(datW, polygon(c(  iq[hOrd],          iq[rev(hOrd)]),
                   c(pred[hOrd, "lwr"], pred[rev(hOrd), "upr"]),
                   border=NA, col=rgb(0.7, 0.7, 0.7, 0.6)))
abline(fitI, col="blue")
points(verbal ~ iq, data=means, col="red", pch=4, lwd=3, cex=1.5)
legend(x="bottomright", legend=c("Daten", "Zentroid", "Vorhersage", "Vertrauensbereich"),
       pch=c(20, 4, NA, NA), lty=c(NA, NA, 1, 1), lwd=c(NA, NA, 1, 8),
       col=c("black", "red", "blue", "gray"))

##### 7.2.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(sumRes <- summary(fitI))
## options(show.signif.stars=FALSE)
confint(fitI)

#####--------------------------------------------------------------------------*
##### 7.3 Multiple lineare Regression ----
##### 7.3.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(fitIA <- lm(verbal ~ iq + attention, data=datW))
fitScl <- lm(scale(verbal) ~ scale(iq) + scale(attention), data=datW)
summary(fitScl)

library(car)
scatter3d(verbal ~ iq + attention, data=datW, fill=FALSE)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

##### 7.3.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitIAE <- lm(verbal ~ iq + attention + education, data=datW)
anova(fitI, fitIAE)

#####--------------------------------------------------------------------------*
##### 7.4 Regressionsmodelle auf andere Daten anwenden ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
newIQ <- c(107, 115, 92, 103)
newDf <- data.frame(iq=newIQ)
predict(fitI, newDf, interval="prediction", level=0.95)

#####--------------------------------------------------------------------------*
##### 7.5 Kreuzvalidierung von Regressionsmodellen ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
glmFit <- glm(verbal ~ iq + attention, data=datW,
              family=gaussian(link="identity"))

library(boot)
kfCV <- cv.glm(datW, glmfit=glmFit, K=5)
kfCV$delta

#####--------------------------------------------------------------------------*
##### 7.6 Regressionsdiagnostik ----
##### 7.6.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Xpred <- with(datW, cbind(iq, attention, education))
Xz    <- scale(Xpred)
summary(Xz)

fitIAE <- lm(verbal ~ iq + attention + education, data=datW)
h      <- hatvalues(fitIAE)
summary(h)

inflRes <- influence.measures(fitIAE)
summary(inflRes)

## ggplot2
library(ggplot2)
library(dplyr)
library(tidyr)
d_XzL <- datW |>
    select(iq, attention, education) |>
    mutate(iq=scale(iq),
           attention=scale(attention),
           education=scale(education)) |>
    pivot_longer(cols=everything(),
                 names_to="Variable",
                 values_to="Value")

ggplot(d_XzL, aes(x=Variable, y=Value)) +
    geom_boxplot() +
    theme_classic()

d_hatv <-data.frame(h=h,
                    idx=seq_along(h))
ggplot(d_hatv, aes(x=idx, y=h, ymin=0, ymax=h)) +
    geom_point(size=3) +
    geom_linerange(size=1) +
    ggtitle("Hebelwerte") +
    theme_classic()

## base R
boxplot(Xz, main="Verteilung der z-standardisierten Prädiktoren")
plot(h, type="h", main="Hebelwerte")

##### 7.6.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Estnd <- rstandard(fitIAE)
Estud <- rstudent(fitIAE)
shapiro.test(Estud)

## ggplot2
library(ggplot2)
d_Estud <- data.frame(Estud=Estud,
                      fit=fitted(fitIAE))

ggplot(d_Estud, aes(x=Estud)) +
    geom_histogram(aes(y=..density..), bins=20) +
    geom_function(fun=dnorm, args=list(mean=0, sd=1), color="red", size=1) +
    ggtitle("Histogramm stud. Residuen") +
    theme_classic()

ggplot(d_Estud, aes(sample=Estud)) +
    geom_qq(size=3) +
    geom_qq_line(color="blue", size=1) +
    ggtitle("Q-Q-Plot stud. Residuen") +
    xlab("Theoretische Quantile") +
    ylab("Beobachtete Quantile") +
    theme_classic()

ggplot(d_Estud, aes(x=fit, y=Estud)) +
    geom_point(size=3) +
    geom_hline(yintercept=0, color="red", size=1) +
    ggtitle("Spread-Level-Plot") +
    xlab("Vorhersage") +
    ylab("studentisierte Residuen") +
    theme_classic()

## base R
hist(Estud, main="Histogramm stud. Residuen", freq=FALSE)
curve(dnorm(x, mean=0, sd=1), col="red", lwd=2, add=TRUE)

qqnorm(Estud, main="Q-Q-Plot stud. Residuen")
qqline(Estud, col="red", lwd=2)

plot(fitted(fitIAE), Estud, pch=20, ylab="studentisierte Residuen",
     xlab="Vorhersage", main="Spread-Level-Plot")
abline(h=0, col="red", lwd=2)

##### 7.6.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
with(datW, round(cor(cbind(iq, attention, education)), 2))

library(car)
vif(fitIAE)

fitScl <- lm(scale(verbal) ~ scale(iq) + scale(attention) + scale(education),
             data=datW)
kappa(fitScl, exact=TRUE)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

#####--------------------------------------------------------------------------*
##### 7.7 Partialkorrelation und Semipartialkorrelation ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
verbal.E <- residuals(lm(verbal ~ education, data=datW))
iq.E     <- residuals(lm(iq ~ education, data=datW))
cor(verbal.E, iq.E)

verbal.EA <- residuals(lm(verbal ~ education + attention, data=datW))
iq.EA     <- residuals(lm(iq ~ education + attention, data=datW))
cor(verbal.EA, iq.EA)

cor(datW$verbal, iq.E)

#####--------------------------------------------------------------------------*
##### 7.8 Logistische Regression ----
##### 7.8.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(glmFit <- glm(attFac ~ verbal, family=binomial(link="logit"), data=datW))

exp(coef(glmFit))
exp(confint(glmFit))

##### 7.8.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
library(DescTools)
PseudoR2(glmFit, which=c("McFadden", "Nagelkerke"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
detach(package:DescTools)

##### 7.8.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Phat   <- fitted(glmFit)
thresh <- 0.5
facHat <- cut(Phat, breaks=c(-Inf, thresh, Inf), labels=c("lo", "hi"))
cTab   <- xtabs(~ attFac + facHat, data=datW)
addmargins(cTab)
(CCR   <- sum(diag(cTab)) / sum(cTab))

dfNew <- data.frame(verbal=rnorm(3, mean=4, sd=3))
predict(glmFit, newdata=dfNew, type="response")

##### 7.8.4 Signifikanztests für Parameter und Modell ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
summary(glmFit)

glm0 <- glm(attFac ~ 1,      family=binomial(link="logit"), data=datW)
anova(glm0, glmFit, test="Chisq")

####*--------------------------------------------------------------------------*
##### 8 t-Tests und Varianzanalysen ----
#####--------------------------------------------------------------------------*

##### 8.1 Test auf Normalverteilung ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
ks.test(datW$iq, "pnorm", mean=100, sd=15, alternative="two.sided")
shapiro.test(datW$iq)

#####--------------------------------------------------------------------------*
##### 8.2 Levene-Test auf Varianzhomogenität ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
library(car)
leveneTest(verbal.pre ~ group, data=datW)
leveneTest(verbal.pre ~ sex * group, data=datW)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

#####--------------------------------------------------------------------------*
##### 8.3 t-Tests ----
##### 8.3.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
muH0 <- 100
t.test(datW$iq, alternative="two.sided", mu=muH0)

##### 8.3.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
t.test(attention.post ~ sex, data=datW, alternative="less", var.equal=TRUE)
t.test(attention.post ~ sex, data=datW, alternative="less", var.equal=FALSE)

##### 8.3.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
with(datW, t.test(attention.pre, attention.post, paired=TRUE))

datSub    <- subset(datL, phase %in% c("pre", "post"), select=c(ID, attention, phase))
datUnique <- droplevels(unique(datSub))
t.test(attention ~ phase, data=datUnique, paired=TRUE)

#####--------------------------------------------------------------------------*
##### 8.4 Einfaktorielle Varianzanalyse (CR-p) ----
##### 8.4.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
CRp <- aov(DV.fup.1 ~ group, data=datW)
summary(CRp)
model.tables(CRp, type="means")

##### 8.4.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(anovaCRp <- anova(lm(DV.fup.1 ~ group, data=datW)))
anovaCRp["Residuals", "Sum Sq"]

##### 8.4.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
CRp   <- aov(DV.fup.1 ~ group, data=datW)
Estud <- rstudent(CRp)

## ggplot2
d_crp <- datW |>
    mutate(fit=fitted(CRp),
           Estud=Estud)

ggplot(d_crp, aes(x=fit, y=Estud)) +
    geom_point(size=3) +
    geom_hline(yintercept=0, size=1) +
    ggtitle("Stud. Residuen gegen Vorhersage") +
    xlab("Vorhersage") +
    ylab("Stud. Residuen") +
    theme_classic()

ggplot(d_crp, aes(x=group, y=Estud)) +
    geom_boxplot() +
    ggtitle("Residuen vs. Stufen") +
    xlab("Gruppe") +
    ylab("Stud. Residuen") +
    theme_classic()

ggplot(d_crp, aes(sample=Estud)) +
    geom_qq(size=3) +
    geom_qq_line(color="blue", size=1) +
    ggtitle("Q-Q-Plot stud. Residuen") +
    xlab("Theoretische Quantile") +
    ylab("Beobachtete Quantile") +
    theme_classic()

## base R
plot(fitted(CRp), Estud, pch=19, xlab="Vorhersage", ylab="stud. Residuen",
     main="stud. Residuen gegen Vorhersage")
abline(h=0, col="gray60", lwd=2)

plot(Estud ~ CRp$model$group, main="Residuen vs. Stufen")

qqnorm(Estud, pch=19)
qqline(Estud, col="gray60", lwd=2)

##### 8.4.4 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
with(datW, pairwise.t.test(DV.fup.1, group,
     p.adjust.method="bonferroni"))

##### 8.4.5 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(tHSD <- TukeyHSD(CRp))
plot(tHSD)

#####--------------------------------------------------------------------------*
##### 8.5 Einfaktorielle Varianzanalyse mit abhängigen Gruppen ----
##### 8.5.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
datSub    <- subset(datL, select=c(ID, attention, phase))
datUnique <- unique(datSub)
summary(aov(attention ~ phase + Error(ID/phase), data=datUnique))

##### 8.5.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitRB <- lm(cbind(attention.pre, attention.post, attention.fup) ~ 1, data=datW)
inRB  <- data.frame(phase=gl(3, 1, labels=c("pre", "post", "fup")))

library(car)
AnovaRB <- Anova(fitRB, idata=inRB, idesign=~phase)
summary(AnovaRB, multivariate=FALSE, univariate=TRUE)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

#####--------------------------------------------------------------------------*
##### 8.6 Zweifaktorielle Varianzanalyse ----
##### 8.6.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
summary(aov(DV.fup.1 ~ sex*group, data=datW))

## ggplot2
library(dplyr)
library(tidyr)
library(ggplot2)
datW_M <- datW |>
    group_by(sex, group) |>
    summarise(DV_fup_1=mean(DV.fup.1))

datW_marginalM <- datW |>
    mutate(M=mean(DV.fup.1)) |>
    group_by(sex) |>
    mutate(M_sex=mean(DV.fup.1)) |>
    ungroup() |>
    group_by(group) |>
    mutate(M_group=mean(DV.fup.1),
           sex=as.character(sex),
           group=as.character(group)) |>
    select(sex, group, M, M_sex, M_group) |>
    unique() # |>
    # reshape(direction="long",
    #         varying=list(c("sex", "group"), c("M_sex", "M_group")),
    #         v.names=c("Factor", "M_marginal"))

ggplot(datW_M, aes(x=sex, y=DV_fup_1, group=group, color=group)) +
    geom_point(size=3) +
    geom_line(size=1) +
    ggtitle("Mittelwertsverläufe") +
    ylab("Mittelwert DV.fup.1") +
    theme_classic()

## base R
plot.design(DV.fup.1 ~ sex*group, data=datW, main="Randmittelwerte")
with(datW, interaction.plot(sex, group, DV.fup.1, main="Mittelwertsverläufe", col=1:3, lwd=2))

##### 8.6.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitIII <- lm(DV.fup.1 ~ sex*group, data=datW,
             contrasts=list(sex=contr.sum, group=contr.sum))

library(car)
Anova(fitIII, type="III")

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

#####--------------------------------------------------------------------------*
##### 8.7 Zweifaktorielle Varianzanalyse mit zwei Intra-Gruppen Faktoren ----
##### 8.7.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
summary(aov(DV ~ phase*hour + Error(ID/(phase*hour)), data=datL))

##### 8.7.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitRBF <- lm(cbind(DV.pre.1,  DV.pre.2,  DV.pre.3,  DV.pre.4,  DV.pre.5,
                   DV.post.1, DV.post.2, DV.post.3, DV.post.4, DV.post.5,
                   DV.fup.1,  DV.fup.2,  DV.fup.3,  DV.fup.4,  DV.fup.5) ~ 1,
             data=datW)
inRBF <- expand.grid(hour=gl(5, 1), phase=gl(3, 1, labels=c("pre", "post", "fup")))

library(car)
AnovaRBF <- Anova(fitRBF, idata=inRBF, idesign=~phase*hour)
summary(AnovaRBF, multivariate=FALSE, univariate=TRUE)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

#####--------------------------------------------------------------------------*
##### 8.8 Zweifaktorielle Varianzanalyse mit Split-Plot-Design ----
##### 8.8.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
datSub    <- subset(datL, select=c(ID, group, attention, phase))
datUnique <- unique(datSub)
# Quadratsummen Typ I
summary(aov(attention ~ group*phase + Error(ID/phase), data=datUnique))

##### 8.8.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitSPF <- lm(cbind(attention.pre, attention.post, attention.fup) ~ group, data=datW)
inSPF  <- data.frame(phase=gl(3, 1, labels=c("pre", "post", "fup")))

library(car)
AnovaSPF <- Anova(fitSPF, idata=inSPF, idesign=~phase)
# Quadratsummen Typ II
summary(AnovaSPF, multivariate=FALSE, univariate=TRUE)

# Quadratsummen Typ III
options(contrasts=c(unordered="contr.sum", ordered="contr.poly"))
fitIII   <- lm(cbind(attention.pre, attention.post, attention.fup) ~ group, data=datW)
AnovaIII <- Anova(fitIII, idata=inSPF, idesign=~phase, type="III")
summary(AnovaIII, multivariate=FALSE, univariate=TRUE)

# Kontrastcodierung zu R-default zurückstellen
options(contrasts=c(unordered="contr.treatment", ordered="contr.poly"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

##### 8.8.3 Erweiterung auf dreifaktorielles SPF-p.qr Design ----
##### 8.8.3.1 Univariat formulierte Auswertung mit aov() ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
summary(aov(DV ~ group*phase*hour + Error(ID/(phase*hour)), data=datL))

##### 8.8.3.2 Multivariat formulierte Auswertung ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitSPFp.qr <- lm(cbind(DV.pre.1, DV.pre.2, DV.pre.3, DV.pre.4, DV.pre.5,
                       DV.post.1, DV.post.2, DV.post.3, DV.post.4, DV.post.5,
                       DV.fup.1, DV.fup.2, DV.fup.3, DV.fup.4, DV.fup.5) ~ group,
                 data=datW)
inSPFp.qr  <- expand.grid(hour=gl(5, 1), phase=gl(3, 1, labels=c("pre", "post", "fup")))

library(car)
## Quadratsummen Typ II
AnovaSPFp.qr <- Anova(fitSPFp.qr, idata=inSPFp.qr, idesign=~phase*hour)
summary(AnovaSPFp.qr, multivariate=FALSE, univariate=TRUE)

## Quadratsummen Typ III
options(contrasts=c(unordered="contr.sum", ordered="contr.poly"))
fitIII   <- lm(cbind(DV.pre.1, DV.pre.2, DV.pre.3, DV.pre.4, DV.pre.5,
                       DV.post.1, DV.post.2, DV.post.3, DV.post.4, DV.post.5,
                       DV.fup.1, DV.fup.2, DV.fup.3, DV.fup.4, DV.fup.5) ~ group,
               data=datW)
AnovaIII <- Anova(fitIII, idata=inSPFp.qr, idesign=~phase*hour, type="III")
summary(AnovaIII, multivariate=FALSE, univariate=TRUE)

## Kontrastcodierung zu R-default zurückstellen
options(contrasts=c(unordered="contr.treatment", ordered="contr.poly"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

##### 8.8.4 Erweiterung auf dreifaktorielles SPF-pq.r Design ----
##### 8.8.4.1 Univariat formulierte Auswertung mit aov() ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
datSub    <- subset(datL, select=c(ID, sex, group, attention, phase))
datUnique <- unique(datSub)
# Quadratsummen Typ I
summary(aov(attention ~ sex*group*phase + Error(ID/phase), data=datUnique))

##### 8.8.4.2 Multivariat formulierte Auswertung ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitSPFpq.r <- lm(cbind(attention.pre, attention.post, attention.fup) ~ sex*group, data=datW)
inSPFpq.r  <- data.frame(phase=gl(3, 1, labels=c("pre", "post", "fup")))

library(car)
## Quadratsummen Typ II
AnovaSPFpq.r <- Anova(fitSPFpq.r, idata=inSPFpq.r, idesign=~phase)
summary(AnovaSPFpq.r, multivariate=FALSE, univariate=TRUE)

## Quadratsummen Typ III
options(contrasts=c(unordered="contr.sum", ordered="contr.poly"))
fitIII   <- lm(cbind(attention.pre, attention.post, attention.fup) ~ sex*group, data=datW)
AnovaIII <- Anova(fitIII, idata=inSPFpq.r, idesign=~phase, type="III")
summary(AnovaIII, multivariate=FALSE, univariate=TRUE)

## Kontrastcodierung zu R-default zurückstellen
options(contrasts=c(unordered="contr.treatment", ordered="contr.poly"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

#####--------------------------------------------------------------------------*
##### 8.9 Kovarianzanalyse ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
fitFull <- lm(DV.post ~ group + DV.pre, data=datW)
fitGrp  <- lm(DV.post ~ group,          data=datW)
fitRegr <- lm(DV.post ~         DV.pre, data=datW)

anova(fitGrp)
anova(fitFull)

library(car)
fitFiii <- lm(DV.post ~ group + DV.pre, data=datW,
              contrasts=list(group=contr.sum))
Anova(fitFiii, type="III")

summary(fitFull)

sumRes    <- summary(fitFull)
coeffs    <- coef(sumRes)
iCeptA    <- coeffs[1, 1]
iCeptB    <- coeffs[2, 1] + iCeptA
iCeptCtrl <- coeffs[3, 1] + iCeptA
slopeAll  <- coeffs[4, 1]
xLims     <- c(min(datW$DV.pre), max(datW$DV.pre))
yLims     <- c(min(iCeptA, iCeptB, iCeptCtrl), max(datW$DV.post))
Nj        <- table(datW$group)

## ggplot2
library(ggplot2)
ggplot(datW, aes(x=group, y=DV.pre)) +
    geom_boxplot() +
    ggtitle("Prä-Scores pro Gruppe") +
    theme_classic()

ggplot(datW, aes(x=group, y=DV.post)) +
    geom_boxplot() +
    ggtitle("Post-Scores pro Gruppe") +
    theme_classic()

dat_fit <- data.frame(slope=slopeAll,
                      intercept=c(iCeptA, iCeptB, iCeptCtrl),
                      group=c("A", "B", "ctrl"))

ggplot(datW, aes(x=DV.pre, y=DV.post,
                 shape=group, color=group, group=group)) +
    geom_point(size=3) +
    geom_abline(data=dat_fit, aes(intercept=intercept, slope=slope,
                                  color=group), size=1) +
    theme_classic()

## base R
plot(DV.pre  ~ group, data=datW, main="Prä-Scores pro Gruppe")
plot(DV.post ~ group, data=datW, main="Post-Scores pro Gruppe")

plot(DV.post ~ DV.pre, data=datW, xlim=xLims, ylim=yLims,
     pch=rep(c(3, 17, 19), Nj), col=rep(c("red", "green", "blue"), Nj),
     main="Rohdaten und Regressionsgerade pro Gruppe")
legend(x="topleft", legend=c("A", "B", "Ctrl"), pch=c(3, 17, 19),
       col=c("red", "green", "blue"))
abline(iCeptA,    slopeAll, col="red")
abline(iCeptB,    slopeAll, col="green")
abline(iCeptCtrl, slopeAll, col="blue")

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:car))

####*--------------------------------------------------------------------------*
##### 9 Nonparametrische Methoden ----
#####--------------------------------------------------------------------------*

#####--------------------------------------------------------------------------*
##### 9.1 Häufigkeiten kategorialer Variablen analysieren ----
##### 9.1.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(tab <- xtabs(~ sex, data=datW))
pH0  <- 0.5
binom.test(tab, p=pH0, alternative="greater")

##### 9.1.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(tab  <- xtabs(~ hair, data=datW))
nCats <- length(tab)
pH0   <- rep(1/nCats, nCats)
chisq.test(tab, p=pH0)

##### 9.1.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
cTab <- xtabs(~ hair + eye, data=datW)
addmargins(cTab)
chisq.test(cTab)
chisq.test(cTab, simulate.p.value=TRUE)

##### 9.1.4 ----
total <- c(5000, 5000, 5000)
hits  <- c(585, 610, 539)
prop.test(hits, total)

##### 9.1.5 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
cTab <- xtabs(~ sex + attFac, data=datW)
addmargins(cTab)
fisher.test(cTab, alternative="two.sided")

##### 9.1.6 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
glmFit <- glm(attFac ~ verbal + group, family=binomial, data=datW)
Phat   <- fitted(glmFit)
facHat <- cut(Phat, breaks=c(-Inf, 0.5, Inf), labels=c("lo", "hi"))
cTab   <- xtabs(~ attFac + facHat, data=datW)  # Kontingenztafel
addmargins(cTab)

TN <- cTab[1, 1]
TP <- cTab[2, 2]
FP <- cTab[1, 2]
FN <- cTab[2, 1]

##### 9.1.6.1 ----
(prevalence  <- sum(cTab[2, ])  / sum(cTab))
(sensitivity <- recall <-    TP / (TP+FN))
(specificity <- TN              / (TN+FP))
(relevance   <- precision <- TP / (TP+FP))
(CCR         <- sum(diag(cTab)) / sum(cTab))
(F1val       <- 1 / mean(1 / c(precision, recall)))

##### 9.1.6.2 ----
library(DescTools)
OddsRatio(cTab, conf.level=0.95)
YuleQ(cTab)
RelRisk(cTab)
## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich

try(detach(package:DescTools))

##### 9.1.7 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
library(pROC)
N      <- 100
height <- rnorm(N, 175, 7)
age    <- rnorm(N, 30, 8)
weight <- 0.4*height + 0.3*age + rnorm(N, 0, 3)

wFac <- cut(weight, breaks=c(-Inf, median(weight), Inf),
            labels=c("lo", "hi"))

regDf   <- data.frame(wFac, height, age)
fit_glm <- glm(wFac ~ height + age, family=binomial(link="logit"),
               data=regDf)

(rocRes <- roc(fit_glm$y, fit_glm$fitted.values,
               plot=TRUE, ci=TRUE,
               main="ROC-Kurve",
               xlab="1-Spezifität (TN / (TN+FP))",
               ylab="Sensitivität (TP / (TP+FN))"))

rocCI <- ci.se(rocRes)
plot(rocCI, type="shape")

try(detach(package:pROC))

#####--------------------------------------------------------------------------*
##### 9.2 Maße für Zusammenhang und Übereinstimmung kategorialer Daten ----
##### 9.2.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
with(datW, cor(attention, verbal, method="spearman"))
with(datW, cor(attention, verbal, method="kendall"))

cor.test(~ attention + verbal, data=datW, method="spearman")
cor.test(~ attention + verbal, data=datW, method="kendall", exact=TRUE)

##### 9.2.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
cTab <- xtabs(~ hair + eye, data=datW)
library(DescTools)
Assocs(cTab)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))
try(detach(package:manipulate))

##### 9.2.3 ----
##### 9.2.3.1 ----
categ <- c("V", "N", "P")
lvls  <- factor(categ, levels=categ)
rtr1  <- rep(lvls, c(60, 30, 10))
rtr2  <- rep(rep(lvls, nlevels(lvls)), c(53,5,2, 11,14,5, 1,6,3))
cTab  <- xtabs(~ rtr1 + rtr2)
addmargins(cTab)

library(DescTools)
Agree(cbind(rtr1, rtr2))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))
try(detach(package:manipulate))

##### 9.2.3.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
library(DescTools)
CohenKappa(cTab, conf.level=0.95)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))
try(detach(package:manipulate))

##### 9.2.3.3 ----
nRtr <- 4
nObs <- 6
rtr1 <- c(9, 6, 8, 7, 10, 6)
rtr2 <- c(2, 1, 4, 1,  5, 2)
rtr3 <- c(5, 3, 6, 2,  6, 4)
rtr4 <- c(8, 2, 8, 6,  9, 7)

library(DescTools)
ICC(cbind(rtr1, rtr2, rtr3, rtr4), conf.level=0.95)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:psych))

#####--------------------------------------------------------------------------*
##### 9.3 Tests auf Übereinstimmung von Verteilungen ----
##### 9.3.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
medH0 <- 100
wilcox.test(datW$iq, alternative="greater", mu=medH0, conf.int=TRUE)

##### 9.3.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
wilcox.test(attention.fup ~ sex, data=datW, conf.int=TRUE)

##### 9.3.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
kruskal.test(DV.fup ~ group, data=datW)

##### 9.3.4 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
datSub    <- subset(datL, select=c(ID, attention, phase))
datUnique <- unique(datSub)
friedman.test(attention ~ phase | ID, data=datUnique)

####*--------------------------------------------------------------------------*
##### 10 Multivariate Verfahren ----
#####--------------------------------------------------------------------------*

#####--------------------------------------------------------------------------*
##### 10.1 Hauptkomponentenanalyse ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
X    <- with(datW, cbind(iq, attention))
(pca <- prcomp(X))
head(pca$x, n=3)
summary(pca)

dev.new(width=14, height=7)
eig    <- eigen(cov(X))
eigVal <- eig$values
G      <- eig$vectors
B      <- G %*% diag(sqrt(eigVal))
ctr    <- colMeans(X)
xMat   <- rbind(ctr[1] - B[1, ], ctr[1])
yMat   <- rbind(ctr[2] - B[2, ], ctr[2])
ab1    <- solve(cbind(1, xMat[ , 1]), yMat[ , 1])
ab2    <- solve(cbind(1, xMat[ , 2]), yMat[ , 2])

par(mfrow=c(1, 2), lend=1)
plot(X, xlab="x", ylab="y", pch=20, asp=1, main="Datenwolke und Hauptkomponenten")
abline(coef=ab1, lwd=2, col="gray")
abline(coef=ab2, lwd=2, col="gray")
matlines(xMat, yMat, lty=1, lwd=6, col="blue")
points(ctr[1], ctr[2], pch=16, col="red", cex=3)
legend(x="topleft", legend=c("Daten", "Achsen HK", "Streuungen HK", "Zentroid"),
       pch=c(20, NA, NA, 16), lty=c(NA, 1, 1, NA), lwd=c(NA, 2, 2, NA),
       col=c("black", "gray", "blue", "red"))

Xdot <- scale(X, center=TRUE, scale=FALSE)
Y    <- Xdot %*% G
B    <- G %*% diag(sqrt(eigVal))
H    <- scale(Y)
HB   <- H %*% t(B)
HB1  <- H[ , 1] %*% t(B[ , 1])
repr <- sweep(HB1, 2, ctr, "+")

plot(X, xlab="x", ylab="y", pch=16, asp=1, main="Originaldaten und Approximation")
abline(coef=ab1, lwd=2, col="gray")
abline(coef=ab2, lwd=2, col="gray")
segments(X[ , 1], X[ , 2], repr[ , 1], repr[ , 2])
points(repr, pch=1, lwd=2, col="blue", cex=2)
points(ctr[1], ctr[2], pch=16, col="red", cex=3)
legend(x="topleft", legend=c("Daten", "Achsen HK", "Zentroid", "Approximation"),
       pch=c(20, NA, 16, 1), lty=c(NA, 1, NA, NA), lwd=c(NA, 2, NA, 2),
       col=c("black", "gray", "red", "blue"))

#####--------------------------------------------------------------------------*
##### 10.2 Faktorenanalyse ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
X   <- with(datW, cbind(iq, education, attention, verbal, DV))
(fa <- factanal(X, factors=2, scores="regression"))
head(fa$scores, n=3)

dev.new(width=14, height=7)
par(mfrow=c(1, 2))
Lhat <- fa$loadings
plot(Lhat, xlab="Faktor 1", ylab="Faktor 2",
     xlim=c(-1.1, 1.1), ylim=c(-1.1, 1.1), pch=20, asp=1,
     main="Faktorenanalyse: Ladungen und Faktoren")
abline(h=0, v=0)
arrows(0, 0, c(1, 0), c(0, 1), col="blue", lwd=2)
angles <- seq(0, 2*pi, length.out=200)
circ   <- cbind(cos(angles), sin(angles))
lines(circ)
text(Lhat[ , 1] , Lhat[ , 2]+0.08, labels=paste("Var", 1:5))
legend(x="bottomright", legend=c("Ladungen", "Faktoren"),
       pch=c(20, NA), lty=c(NA, 1), col=c("black", "blue"))

plot(eigen(cor(X))$values, type="b", ylab="Eigenwert", main="Scree-Plot")
abline(h=1)

#####--------------------------------------------------------------------------*
##### 10.3 Multivariate multiple Regression ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Y <- with(datW, cbind(attention, education))
(fitM <- lm(Y ~ iq + verbal, data=datW))

summary(manova(fitM), test="Hotelling-Lawley")
summary(manova(fitM), test="Wilks")
summary(manova(fitM), test="Roy")
summary(manova(fitM), test="Pillai")

#####--------------------------------------------------------------------------*
##### 10.4 Hotellings T^2 ----
##### 10.4.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Y    <- with(datW, cbind(DV.pre, DV.post, DV.fup))
muH0 <- c(5, 5, 5)

library(DescTools)
HotellingsT2Test(Y, mu=muH0)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))
try(detach(package:manipulate))

##### 10.4.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Y   <- with(datW, cbind(DV.pre, DV.post, DV.fup))
sex <- with(datW, sex)

library(DescTools)
HotellingsT2Test(Y ~ sex)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))
try(detach(package:manipulate))

#####--------------------------------------------------------------------------*
##### 10.5 Multivariate Varianzanalyse ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Y <- with(datW, cbind(DV.pre, DV.post, DV.fup))
summary(manova(Y ~ group, data=datW), test="Wilks")
summary(manova(Y ~ group, data=datW), test="Roy")
summary(manova(Y ~ group, data=datW), test="Pillai")
summary(manova(Y ~ group, data=datW), test="Hotelling-Lawley")

#####--------------------------------------------------------------------------*
##### 10.6 Diskriminanzanalyse ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
library(MASS)
(ldaRes <- lda(group ~ DV.pre + DV.post + DV.fup, data=datW))

ldaPred <- predict(ldaRes, datW)
head(ldaPred$x, n=3)
head(ldaPred$class, n=3)

cTab <- xtabs(~ datW$group + ldaPred$class)
addmargins(cTab)
sum(diag(cTab)) / sum(cTab)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:MASS))

####*--------------------------------------------------------------------------*
##### 11 Diagramme erstellen ----
#####--------------------------------------------------------------------------*

#### 11.1 Grundprinzip ----
##### 11.1.1 Grundschicht ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")

library(ggplot2)
ggplot(datW, aes(x=DV.pre, y=DV.post, colour=sex, shape=group))

##### 11.1.2 Diagramme speichern ----

####*--------------------------------------------------------------------------*
#### 11.2 Diagrammtypen ----
##### 11.2.1 Punkt-, Streu und Liniendiagramm ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
head(datW, n=3)
head(datL, n=3)

library(dplyr)
ggplot(datW, aes(x=DV.pre, y=DV.post, colour=sex, shape=group)) +
        geom_point(size=3)

groupM <- datW  |>
    group_by(sex, group) |>
    summarise(DV.pre=mean(DV.pre)) |>
    ungroup()

groupM
ggplot(groupM, aes(x=group, y=DV.pre, color=sex, shape=sex, group=sex)) +
    geom_point() +
    geom_line()

##### 11.2.2 Säulendiagramm ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
ggplot(datW, aes(x=hair, group=sex, fill=sex)) +
    geom_bar(stat="count", position=position_stack())

ggplot(datW, aes(x=hair, group=sex, fill=sex)) +
    geom_bar(stat="count",
             aes(y=(..count..) / sum(..count..)),
             position=position_fill())

ggplot(datW, aes(x=hair, group=sex, fill=sex)) +
    geom_bar(stat="count",
             aes(y=(..count..) / sum(..count..)),
             position=position_dodge())

##### 11.2.3 Histogramm ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
ggplot(datW, aes(x=attention)) +
    geom_histogram(bins=15)

ggplot(datW, aes(x=attention)) +
    geom_histogram(aes(y=..density..), bins=15) +
    geom_density(color="darkgrey", fill="grey", alpha=0.6)

##### 11.2.4 Boxplot ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
ggplot(datW, aes(y=DV.pre)) +
    geom_boxplot()

ggplot(datW, aes(x=sex, y=DV.pre, fill=sex)) +
    geom_boxplot()

library(ggbeeswarm)
ggplot(datW, aes(x=sex, y=DV.pre, fill=sex)) +
    geom_boxplot(outlier.shape=NULL) +
    geom_beeswarm(size=2, alpha=0.5, cex=2)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:ggbeeswarm))

##### 11.2.5 Q-Q Diagramm ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
ggplot(datW, aes(sample=attention)) +
    geom_qq() +
    geom_qq_line(color="blue")

parL <- list(df=10, ncp=0)
ggplot(datW, aes(sample=attention)) +
     geom_qq(distribution=qt, dparams=parL) +
     geom_qq_line(color="blue", distribution=qt, dparams=parL)

####*--------------------------------------------------------------------------*
#### 11.3 Bedingte Diagramme in Panels darstellen ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
ggplot(datW, aes(x=sex, y=attention, fill=sex)) +
    geom_boxplot() +
    facet_grid(. ~ group)

library(ggbeeswarm)
ggplot(datL, aes(x=hour, y=DV, color=group)) +
    geom_beeswarm() +
    facet_wrap(~ phase, ncol=2, scales="free")

####*--------------------------------------------------------------------------*
#### 11.4 Diagrammelemente hinzufügen ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
library(dplyr)
groupMSDN <- datW |>
    group_by(sex, group) |>
    summarise(attn.M=mean(attention),
              attn.SD=sd(attention),
              Freq=n()) |>
    mutate(SEMlo=attn.M - attn.SD/sqrt(Freq),
           SEMup=attn.M + attn.SD/sqrt(Freq))

ggplot(groupMSDN, aes(x=group, y=attn.M, ymin=SEMlo, ymax=SEMup,
                  color=sex, shape=sex, group=sex)) +
    geom_point() +
    geom_line() +
    geom_linerange()

ggplot(datW, aes(x=DV.pre, y=DV.post, colour=sex, shape=sex)) +
    geom_hline(aes(yintercept=5), linetype=2) +
    geom_vline(aes(xintercept=5), linetype=2) +
    geom_point(size=3) +
    geom_smooth(method=lm, se=TRUE, size=1.2, fullrange=TRUE) +
    facet_grid(~ group) +
    labs(title="DV.post ~ DV.pre getrennt für Geschlecht+Gruppe") +
    annotate("text", x=3.5, y=3, size=7, label="Annotation")

####*--------------------------------------------------------------------------*
#### 11.5 Diagramme formatieren ----
##### 11.5.1 Elemenposition kontrollieren ----
ggplot(datW, aes(x=attention, fill=sex)) +
    geom_histogram(aes(y=..density..), position=position_dodge(), bins=10)

ggplot(datW, aes(x=hair, group=sex, fill=sex)) +
    geom_bar(stat="count",
             aes(y=(..count..) / sum(..count..)),
             position=position_fill())

##### 11.5.2 Achsen anpassen ----
levels(datW$hair)
levels(datW$hair) <- rev(levels(datW$hair))
ggplot(datW, aes(x=hair, group=sex, fill=sex)) +
    geom_bar(stat="count",
             aes(y=(..count..) / sum(..count..)),
             position=position_fill()) +
    labs(x="Rating category", y="Cumulative relative frequency") +
    guides(x=guide_axis(angle=90))

ggplot(datW, aes(x=attention, y=DV.post, colour=sex, shape=group)) +
    geom_point(size=3) +
    scale_x_continuous(limits=c(0, 60),
                       expand=c(0, 0),
                       breaks=seq(0, 60, by=5)) +
    scale_y_continuous(n.breaks=8) +
    guides(x=guide_axis(angle=90))

ggplot(datW, aes(x=sex, y=DV.fup, fill=sex)) +
        geom_boxplot() +
        coord_flip(ylim=c(0, 8))

##### 11.5.3 Legende ändern ----
## für Daten des Buches diese Datei laden:
# load("data_ggplot.Rdata")
ggplot(datW, aes(x=DV.pre, y=DV.post, colour=sex, shape=sex)) +
    geom_hline(aes(yintercept=5), linetype=2) +
    geom_vline(aes(xintercept=5), linetype=2) +
    geom_point(size=3) +
    geom_smooth(method=lm, se=TRUE, size=1.2, fullrange=TRUE) +
    facet_grid(~ group) +
    labs(title="DV.post ~ DV.pre getrennt für Geschlecht+Gruppe") +
    guides(colour=FALSE) +
    theme(legend.position="bottom")

##### 11.5.4 Farben, Datenpunktsymbole und Linientypen ----
library(colorspace)

ggplot(groupM, aes(x=group, y=DV.pre, color=sex, shape=sex, group=sex)) +
    geom_point(size=8, stroke=2) +
    geom_line(size=2, linetype="dashed") +
    scale_shape_discrete(solid=FALSE)

ggplot(datW, aes(x=DV.fup, fill=group)) +
    geom_histogram(aes(y=..density..), bins=15, alpha=0.5) +
    geom_density(alpha=0.7) +
    scale_fill_discrete_qualitative()

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:colorspace))

##### 11.5.5 Aussehen im Detail verändern ----
library(colorspace)

ggplot(datW, aes(x=DV.fup, fill=group)) +
    geom_histogram(aes(y=..density..), bins=15, alpha=0.5) +
    geom_density(alpha=0.7) +
    scale_fill_discrete_qualitative() +
    theme_bw()

ggplot(datW, aes(x=sex, y=DV.fup, fill=sex)) +
    geom_boxplot() +
    coord_flip(ylim=c(0, 8)) +
    theme_minimal()

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:ggplot2))
try(detach(package:colorspace))

####*--------------------------------------------------------------------------*
##### 12 R als Programmiersprache ----
#####--------------------------------------------------------------------------*

#####--------------------------------------------------------------------------*
##### 12.1 Kontrollstrukturen ----
##### 12.1.1 ----
(x <- round(rnorm(1, 100, 15)))
if(x > 100) { cat("x is", x, "(greater than 100)\n") }

x <- round(rnorm(1, 100, 15))
if (x > 100) {
    cat("x is", x, "(greater than 100)\n")
} else {
    cat("x is", x, "(100 or less)\n")
}

##### 12.1.2 ----
ABC <- c("Alfa", "Bravo", "Charlie", "Delta")
for(i in ABC) { print(i) }

x <- 37
y <- 10
while(x >= y) { x <- x-y }
x

for(i in 1:10) {
    if((i %% 2) != 0) { next  }
    if((i %% 8) == 0) { break }
    print(i)
}

#####--------------------------------------------------------------------------*
##### 12.2 Eigene Funktionen erstellen ----
##### 12.2.1 ----
.First <- function() {
    library(car)
    print("Have a nice day!")
}

##### 12.2.2 ----
##### 12.2.3 ----
methods("fligner.test")

##### 12.2.4 Funktionen analysieren ----
sd
base::mean
boot:::basic.ci

get("fligner.test")
methods("fligner.test")
getS3method("fligner.test", "default")

##### 12.2.5 ----
system.time(solve(matrix(sample(1:100, 200^2, replace=TRUE), nrow=200)))
