#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschläger <contact [at] dwoll.de>
## Befehle zum Buch "R kompakt" (Springer)
## http://www.dwoll.de/r/
## Stand: 21.08.21
##
## Vgl. Abschnitt 2.2, um die folgenden Befehle auszuführen.
##
## Befehle, die eine Benutzerinteraktion notwendig machen, oder
## nicht erwünschte Nebenwirkungen haben, sind auskommentiert.
##
## Um alle Teile des Skripts ausführen zu können, müssen die Pakete
## DescTools, rgl samt Abhängigkeiten installiert sein
pWants <- c("DescTools", "rgl")
pHas   <- pWants %in% rownames(installed.packages())
if(any(!pHas)) install.packages(pWants[!pHas], repos="https://cran.rstudio.com/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

load(url("http://www.dwoll.de/r/dat.Rdata"))
head(subW)                      # gib die ersten Zeilen von subW aus

####*---------------------------------------------------------------------------
##### 4 Datensätze ----
#####---------------------------------------------------------------------------

## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
head(myDf1)

#####---------------------------------------------------------------------------
##### 4.1 Variablen umbenennen ---
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
names(myDf1)
names(myDf1)[3] <- "fac"
names(myDf1)
names(myDf1)[names(myDf1) == "fac"] <- "group"
names(myDf1)

#####---------------------------------------------------------------------------
##### 4.2 Teilmengen von Daten auswählen ----
##### 4.2.1 ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
subset(myDf1, select=c(group, IQ))
subset(myDf1, select=c(-sex, -IQ))
(colIdx <- grep("^i.+", names(myDf1), ignore.case=TRUE))
subset(myDf1, select=colIdx)

###### 4.2.2 ----
subset(myDf1, sex == "f")
subset(myDf1, id == rating)
subset(myDf1, (sex == "m") & (rating > 2))
subset(myDf1, (IQ < 90) | (IQ > 110))
subset(myDf1, group %in% c("CG", "WL"))

#####---------------------------------------------------------------------------
##### 4.3 Variablen entfernen, hinzufügen und transformieren ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
dfTemp <- myDf1
dfTemp$group <- NULL
head(dfTemp, n=3)

dfTemp[c("sex", "IQ")] <- list(NULL)
head(dfTemp, n=3)

married <- sample(c(TRUE, FALSE), nrow(myDf1), replace=TRUE)
myDf2   <- myDf1
myDf2$married1    <- married
myDf2["married2"] <- married
myDf3 <- cbind(myDf1, married)
head(myDf3, n=3)
myDf4 <- transform(myDf3,
                   rSq=rating^2,
                   IQgrp=cut(IQ, breaks=c(0, 100, Inf)))
head(myDf4, n=3)

#####---------------------------------------------------------------------------
##### 4.4 Doppelte und fehlende Werte behandeln ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
myDfDouble <- rbind(myDf1, myDf1[sample(seq_len(nrow(myDf1)), 4), ])
duplicated(myDfDouble) | duplicated(myDfDouble, fromLast=TRUE)
myDfUnique <- unique(myDfDouble)
any(duplicated(myDfUnique))

myDfNA           <- myDf1
myDfNA$IQ[2]     <- NA
myDfNA$rating[3] <- NA

is.na(myDfNA)[1:3, ]
apply(myDfNA, 2, anyNA)
apply(myDfNA, 2, function(x) { sum(is.na(x)) })
apply(myDfNA, 2, function(x) { length(na.omit(x)) })

complete.cases(myDfNA)
table(complete.cases(myDfNA))
subset(myDfNA, !complete.cases(myDfNA))
head(na.omit(myDfNA), n=4)

#####---------------------------------------------------------------------------
##### 4.5 Datensätze sortieren ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
(idx1 <- order(myDf1$rating))
myDf1[idx1, ]
(idx2 <- order(myDf1$group, myDf1$IQ))
myDf1[idx2, ]

#####---------------------------------------------------------------------------
##### 4.6 Datensätze aufteilen ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
split(myDf1, myDf1$group)
split(myDf1, list(myDf1$sex, myDf1$group))

#####---------------------------------------------------------------------------
##### 4.7 Datensätze zeilen- oder spaltenweise verbinden ----
(dfNew <- data.frame(id=13:15,
                     group=c("CG", "WL", "T"),
                     sex=c("f", "f", "m"),
                     age=c(18, 31, 21),
                     IQ=c(116, 101, 99),
                     rating=c(4, 4, 1)))
dfComb <- rbind(myDf1, dfNew)
dfComb[11:15, ]

#####---------------------------------------------------------------------------
##### 4.8 Datensätze zusammenführen ----
(IDDV <- data.frame(ID=factor(rep(1:3, each=2)),
                    DV=round(rnorm(6, 100, 15))))

(IV <- data.frame(ID=factor(1:3),
                  IV=factor(c("A", "B", "A")),
                  sex=factor(c("f", "f", "m"))))

merge(IDDV, IV)

(dfA <- data.frame(ID=1:4,
                   initials=c("AB", "CD", "EF", "GH"),
                   IV1=c("-", "-", "+", "+"),
                   DV1=c(10, 10, 11, 14)))

(dfB <- data.frame(ID_mod=1:4,
                   initials=c("AB", "CD", "EF", "GH"),
                   IV2=c("A", "B", "A", "B"),
                   DV2=c(91, 89, 92, 79)))

merge(dfA, dfB, by.x="ID", by.y="ID_mod")

(dfC <- data.frame(ID=3:6,
                   initials=c("EF", "GH", "IJ", "KL"),
                   IV2=c("A", "B", "A", "B"),
                   DV2=c(92, 79, 101, 81)))
merge(dfA, dfC)
merge(dfA, dfC,             all.y=TRUE)
merge(dfA, dfC, all.x=TRUE, all.y=TRUE)

#####---------------------------------------------------------------------------
##### 4.9 Organisationsform einfacher Datensätze ändern ----
vec1  <- sample(1:10, 3, replace=TRUE)
vec2  <- sample(1:10, 2, replace=TRUE)
vec3  <- sample(1:10, 1, replace=TRUE)
(lTmp <- list(cond1=vec1, cond2=vec2, cond3=vec3))
(res  <- stack(lTmp))
str(res)
unstack(res)

Nj        <- 3
res$IVnew <- factor(sample(rep(c("A", "B"), Nj), 2*Nj, replace=FALSE))
res$DVnew <- sample(100:200, 2*Nj)
unstack(res, form=DVnew ~ IVnew)

#####---------------------------------------------------------------------------
##### 4.10 Organisationsform einfacher Datensätze ändern ----
###### 4.10.1 ----
Nj    <- 2
P     <- 2
Q     <- 3
id    <- 1:(P*Nj)
DV_t1 <- round(rnorm(P*Nj, -1, 1), 2)
DV_t2 <- round(rnorm(P*Nj,  0, 1), 2)
DV_t3 <- round(rnorm(P*Nj,  1, 1), 2)
IVbtw <- factor(rep(c("A", "B"), Nj))
(dfW1 <- data.frame(id, IVbtw, DV_t1, DV_t2, DV_t3))

idL    <- rep(id, Q)
DVl    <- c(DV_t1, DV_t2, DV_t3)
IVwth  <- factor(rep(1:3, each=P*Nj))
IVbtwL <- rep(IVbtw, times=Q)
dfL1a  <- data.frame(id=idL, IVbtw=IVbtwL, IVwth=IVwth, DV=DVl)
dfL1a[order(dfL1a$id), ]

dfL1b <- reshape(dfW1, varying=c("DV_t1", "DV_t2", "DV_t3"),
                 direction="long", idvar=c("id", "IVbtw"), v.names="DV")
head(dfL1b[order(dfL1b$id), ])
reshape(dfL1b, v.names="DV", timevar="time", idvar=c("id", "IVbtw"),
        direction="wide")

#####---------------------------------------------------------------------------
###### 4.10.2 ----
Nj   <- 4
id   <- 1:Nj
t_11 <- round(rnorm(Nj,  8, 2), 2)
t_12 <- round(rnorm(Nj, 10, 2), 2)
t_21 <- round(rnorm(Nj, 13, 2), 2)
t_22 <- round(rnorm(Nj, 15, 2), 2)
t_31 <- round(rnorm(Nj, 13, 2), 2)
t_32 <- round(rnorm(Nj, 15, 2), 2)
(dfW2 <- data.frame(id, t_11, t_12, t_21, t_22, t_31, t_32))

(dfL2_IV1 <- reshape(dfW2, varying=list(c("t_11", "t_21", "t_31"),
                                        c("t_12", "t_22", "t_32")),
                     direction="long", timevar="IV1", idvar="id",
                     v.names=c("IV2-1", "IV2-2")))

dfL2_IV1_IV2 <- reshape(dfL2_IV1, varying=c("IV2-1", "IV2-2"),
                        direction="long", timevar="IV2", idvar=c("id", "IV1"),
                        v.names="DV")
head(dfL2_IV1_IV2)

dfW2_IV2 <- reshape(dfL2_IV1_IV2, v.names="DV", timevar="IV2",
                    idvar=c("id", "IV1"), direction="wide")

(dfW2_IV1_IV2 <- reshape(dfW2_IV2, v.names=c("DV.1", "DV.2"),
                         timevar="IV1", idvar="id", direction="wide"))

#####---------------------------------------------------------------------------
##### 4.11 Datensätze getrennt nach Gruppen auswerten und aggregieren ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
(dat_spl <- split(myDf1, myDf1$group))
sapply(dat_spl, function(x) { c(M=mean(x$IQ)) })

(m_sd <- sapply(dat_spl, function(x) { c(M=mean(x$IQ), SD=sd(x$IQ)) }))
t(m_sd)

dat_spl2 <- split(myDf1, list(myDf1$sex, myDf1$group), drop=TRUE)
sapply(dat_spl2, function(x) { c(M=mean(x$IQ), SD=sd(x$IQ)) })

aggr_fun <- function(x) {
  data.frame(Sex=unique(x$sex),
             Group=unique(x$group),
             M=mean(x$IQ),
             SD=sd(x$IQ))
}

m_sdL <- lapply(dat_spl2, aggr_fun)
do.call("rbind", m_sdL)

numDf <- subset(myDf1, select=c(age, IQ))
aggregate(numDf, by=list(sex=myDf1$sex, group=myDf1$group), FUN=mean)
aggregate(cbind(age, IQ) ~ sex + group, FUN=mean, data=myDf1)
aggregate(cbind(age, IQ) ~ 1, FUN=mean, data=myDf1)

d_mean <- aggregate(cbind(age, IQ) ~ sex + group, FUN=mean, data=myDf1)
d_sd   <- aggregate(cbind(age, IQ) ~ sex + group, FUN=sd,   data=myDf1)
merge(d_mean, d_sd, by=c("sex", "group"), suffixes = c(".M",".SD"))

(dat_mat <- aggregate(cbind(age, IQ) ~ sex + group, FUN=function(x) { c(M=mean(x), SD=sd(x)) }, data=myDf1))
dat_mat$age
dat_mat$age[ , "M"]

#####---------------------------------------------------------------------------
##### 4.12 Funktionen auf Variablen anwenden ----
## für Daten des Buches diese Datei laden:
# load("data_myDf1.Rdata")
numDf <- subset(myDf1, select=c(age, IQ, rating))
(myList <- lapply(numDf, mean))
sapply(numDf, range)
(numIdx <- sapply(myDf1, is.numeric))
dataNum <- subset(myDf1, select=numIdx)
head(dataNum)
data.matrix(dataNum)

do.call("c", myList)
work   <- factor(sample(c("home", "office"), 20, replace=TRUE))
hiLo   <- factor(sample(c("hi", "lo"),       20, replace=TRUE))
group  <- factor(sample(c("A", "B"),         20, replace=TRUE))
tab    <- table(work, hiLo, group)
argLst <- list(tab, row.vars="work", col.vars=c("hiLo", "group"))
do.call("ftable", argLst)

#####---------------------------------------------------------------------------
##### 4.13 Funktionen für mehrere Variablen anwenden ----
N     <- 100
x1    <- rnorm(N, 10, 10)
y1    <- rnorm(N, 10, 10)
x2    <- x1 + rnorm(N, 5, 4)
y2    <- y1 + rnorm(N, 10, 4)
myDf2 <- data.frame(x1, y1)
myDf3 <- data.frame(x2, y2)
mapply(t.test, myDf2, myDf3, MoreArgs=list(alternative="less", var.equal=TRUE))

####*---------------------------------------------------------------------------
##### 11 Diagramme erstellen ----
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 11.1 Graphik Devices ----
##### 11.1.1 ----
dev.new(); dev.new()
dev.list()
dev.off()
graphics.off()

#####---------------------------------------------------------------------------
##### 11.1.2 ----
pdf("pdf_test.pdf")
plot(1:10, rnorm(10))
dev.off()

#####---------------------------------------------------------------------------
##### 11.2 Streu- und Liniendiagramme ----
##### 11.2.1 ----
x <- sort(rnorm(20))
y <- 0.4*x + rnorm(20, mean=0, sd=4)

dev.new(width=21, height=14)
par(mfrow=c(2, 3), mar=c(4, 3, 3, 2))
plot(x, y, type="p", xlab=NA, main="type p", cex=1.5)
plot(x, y, type="l", xlab=NA, main="type l", cex=1.5)
plot(x, y, type="b", xlab=NA, main="type b", cex=1.5)
plot(x, y, type="o", xlab=NA, main="type o", cex=1.5)
plot(x, y, type="s", xlab=NA, main="type s", cex=1.5)
plot(x, y, type="h", xlab=NA, main="type h", cex=1.5)

vec1 <- sample(1:10, 200, replace=TRUE)
vec2 <- sample(1:10, 200, replace=TRUE)

dev.new(width=14, height=7)
par(mfrow=c(1, 2))
plot(vec2 ~ vec1, main="Punktwolke")
plot(jitter(vec2) ~ vec1, main="Punktwolke mit jitter")

##### 11.2.2 ----
vec <- seq(from=-2*pi, to=2*pi, length.out=50)
mat <- cbind(2*sin(vec), sin(vec-(pi/4)), 0.5*sin(vec-(pi/2)))
matplot(vec, mat, type="b", xlab=NA, ylab=NA, pch=1:3, main="Sinuskurven")

#####---------------------------------------------------------------------------
##### 11.3 Diagramme formatieren ----
##### 11.3.1 ----
X <- row(matrix(numeric(6*11), nrow=6, ncol=11))
Y <- col(matrix(numeric(6*11), nrow=6, ncol=11))

par(mar=c(1, 1, 4, 2))
plot(0:6, seq(1, 11, length.out=7), type="n", axes=FALSE,
     main="pch Datenpunkt-Symbole und lty Linientypen")
points(X[1:26], Y[1:26], pch=0:25, bg="gray", cex=2)
matlines(X[ , 6:11], Y[ , 6:11], lty=6:1, lwd=2, col="black")
text(X[1:26]-0.3, Y[1:26],    labels=0:25)
text(rep(0.7, 6), Y[1, 6:11], labels=6:1)
text(0, 9, labels="Linientypen für lty", srt=90, cex=1.2)
text(0, 3, labels="Symbole für pch",     srt=90, cex=1.2)

##### 11.3.2 ----
palette("default")
rgb(0, 1, 1)
gray(0.5)
## palette(rainbow(10))

##### 11.3.3 ----

#####---------------------------------------------------------------------------
##### 11.4 Säulen- und Punktdiagramme ----
##### 11.4.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(tab <- xtabs(~ hair, data=datW))

barplot(tab, ylim=c(0, 30), xlab="Hair color", ylab="N", col="gray",
        main="Absolute Häufigkeiten")

##### 11.4.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
(cTab <- xtabs(~ hair + eye, data=datW))

barplot(cTab, beside=FALSE, legend.text=TRUE, xlab="Eye color", ylab="N",
        main="Absolute Häufigkeiten einer Kreuztabelle")
barplot(cTab, beside=TRUE, ylim=c(0, 15), col=c("red", "green", "blue", "gray"),
        legend.text=TRUE, xlab="Eye color", ylab="N",
        main="Absolute Häufigkeiten einer Kreuztabelle")

##### 11.4.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
DV <- datW$DV
IV <- datW$group
Mj <- tapply(DV, IV, mean)

dotchart(DV, gdata=Mj, pch=20, color=(1:3)[unclass(IV)],
         gcolor="blue", groups=IV, xlab="AV", ylab="Gruppen", gpch=16,
         main="Individuelle Ergebnisse + Mittelwerte aus 3 Gruppen")

#####---------------------------------------------------------------------------
##### 11.5 Elemente einem bestehenden Diagramm hinzufügen ----
##### 11.5.1 Punkte ----
xA <- seq(-15, 15, length.out=200)
yA <- sin(xA) / xA

dev.new(width=14, height=7)
par(mfrow=c(1, 2))
plot(xA, yA, type="l", xlab="x", ylab="sinc(x)",
     main="Punkte und Linien einfügen", lwd=1.6)

xB <- seq(-15, 15, length.out=30)
yB <- sin(xB) / xB
points(xB, yB, col="red", pch=16)

##### 11.5.2 Linien ----
yC <- sin(pi * xA) / (pi * xA)
lines(xA, yC, col="blue", type="l", lwd=1.6)
abline(h=0, v=0, col="green", lwd=1.6)

fit  <- lm(attention ~ verbal, data=datW)
Yhat <- fitted(fit)
plot(attention ~ verbal, main="Segmente und Polygone einfügen",
     pch=16, data=datW)
with(datW, segments(x0=verbal, y0=attention, x1=verbal, y1=Yhat))
abline(fit, col="blue", lwd=2)

##### 11.5.3 Polygone ----
pred <- predict(fit, interval="confidence", level=0.95)
fOrd <- with(datW, order(verbal))
with(datW, polygon(c(verbal[fOrd],     verbal[rev(fOrd)]),
                   c(pred[fOrd, "lwr"], pred[rev(fOrd), "upr"]),
                   border=NA, col=rgb(0.7, 0.7, 0.7, 0.6)))

rect(1.5, 50, 2.5, 55, col="lightgray", border=NA)
text(2, 48, labels="Vertrauensbereich")

##### 11.5.4 Funktionsgraphen ----
curve(dnorm(x, mean=1, sd=1), from=-7, to=7, col="blue", lwd=2, xlab=NA, axes=FALSE)
curve((1/(2*sqrt(2*pi))) * exp(-0.5*(((x-0)/2)^2)), add=TRUE, lwd=2, lty=2)

##### 11.5.5 Text ----
title(main="zwei Normalverteilungskurven", sub="Untertitel")
legend(x="topleft", legend=c("N(1, 1)", "N(0, 2)"), col=c("blue", "black"),
       lty=c(1, 2))
text(x=3.6,  y=0.35, labels="Normalverteilung\nN(1, 1)")
text(x=-3.5, y=0.1 , labels="N(0, 2)")

##### 11.5.6 Achsen ----
axis(side=1, at=seq(-6, 6, by=2))
axis(side=2, at=seq(0, 0.4, by=0.1))

##### 11.5.7 Fehlerbalken ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
Mj  <- with(datW, tapply(DV, group, FUN=mean))
Sj  <- with(datW, tapply(DV, group, FUN=sd))
Nj  <- xtabs(~ group, data=datW)
CIwidths <- qt(0.975, df=Nj-1) * Sj / sqrt(Nj)

library(DescTools)
stripchart(DV ~ group, data=datW, method="jitter", xlab="Gruppe",
           main="Rohdaten und Konfidenzintervalle", xaxt="n", col="darkgray",
           ylim=c(3.5, 7.5), pch=16, vert=TRUE)
ErrBars(from=Mj-CIwidths, to=Mj+CIwidths, pos=1:3, length=0.1,
        col="blue", col.pch="blue", lwd=2, pch=19)
axis(side=1, at=1:3, labels=LETTERS[1:3])

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:DescTools))

#####---------------------------------------------------------------------------
##### 11.6 Verteilungsdiagramme ----
##### 11.6.1 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")

dev.new(width=14, height=7)
par(mfrow=c(1, 2))
fromTo <- round(range(datW$iq), -1) + c(-10, 10)
limits <- seq(from=fromTo[1], to=fromTo[2], 5)
hist(datW$iq, freq=FALSE, xlim=fromTo, xlab="IQ", ylab="relative Häufigkeit",
     breaks=limits, main="Histogramm und Normalverteilung")
curve(dnorm(x, mean(datW$iq), sd(datW$iq)), lwd=2, col="blue", add=TRUE)

hist(datW$iq, freq=FALSE, xlim=fromTo, xlab="IQ",
     main="Histogramm und Schätzung der Dichte")
lines(density(datW$iq), lwd=2, col="blue")

##### 11.6.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
boxplot(DV ~ group, data=datW, ylab="Score", col="lightgray",
        main="Boxplots der Scores in 3 Gruppen")

Mj <- with(datW, tapply(DV, group, mean))
points(1:3, Mj, pch=16, cex=2)

##### 11.6.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
dev.new(width=14, height=7)
par(mfrow=c(1, 2))
stripchart(DV ~ group, data=datW, xlab="Score", ylab="Gruppe", pch=1,  col="blue",
           main="Scores in 3 Gruppen", sub="jitter-Methode", method="jitter")
stripchart(DV ~ group, data=datW, xlab="Score", ylab="Gruppe", pch=16, col="red",
           main="Scores in 3 Gruppen", sub="stack-Methode",  method="stack")

##### 11.6.4 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
qqnorm(datW$iq, pch=16)
qqline(datW$iq, col="red", lwd=2)

##### 11.6.5 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
tab <- xtabs(~ eye, data=datW)
par(mar=c(2, 2, 3, 2))
pie(tab, col=1:4, main="Häufigkeiten von Augenfarben")

##### 11.6.6 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
ctrJ <- aggregate(cbind(verbal.pre, verbal.post) ~ sex, FUN=mean, data=datW)
plot(verbal.post ~ verbal.pre, data=datW, pch=c(4, 16)[unclass(sex)], lwd=2,
     col=c("black", "darkgray")[unclass(sex)],
     main="Gemeinsame Verteilung getrennt nach Geschlecht")
points(verbal.post ~ verbal.pre, data=ctrJ, pch=c(4, 16),
       col=c("black", "darkgray"), cex=3, lwd=5)
legend(x="topleft", legend=c("female", "male"), pch=c(4, 16),
       col=c("black", "darkgray"))

#####---------------------------------------------------------------------------
##### 11.7 Multivariate Daten visualisieren ----
##### 11.7.1 ----
X <- Y <- seq(-10, 10, length.out=50)
f <- function(x, y) { r <- sqrt(x^2+y^2); 10 * sin(r)/r }
Z <- outer(X, Y, FUN=f)

contour(X, Y, Z, main="Höhenlinien für 2D Sinc-Funktion")
filled.contour(X, Y, Z, main="Farbige Höhenlinien für 2D Sinc-Funktion")

##### 11.7.2 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
par(mar=c(2, 2, 4, 2) + 0.1)
persp(X, Y, Z, xlab="x", ylab="y", zlab=NA, theta=5, phi=35,
      main="2D Sinc-Funktion")

library(rgl)
vecX <- rep(seq(-10, 10, length.out=10), times=10)
vecY <- rep(seq(-10, 10, length.out=10),  each=10)
vecZ <- vecX*vecY
plot3d(vecX, vecY, vecZ, main="3D Scatterplot", col="blue", type="h", aspect=TRUE)
spheres3d(vecX, vecY, vecZ, col="red", radius=2)
grid3d(c("x", "y+", "z"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern möglich
try(detach(package:rgl))

##### 11.7.3 ----
## für Daten des Buches diese Datei laden:
# load("dat.Rdata")
pairs(subset(datW, select=c(education, iq, attention, verbal)), main="Streudiagramm-Matrix",
      pch=16, col=c("red", "blue")[unclass(datW$sex)])

#####---------------------------------------------------------------------------
##### 11.8 Mehrere Diagramme in einem Graphik-Device darstellen ----
dev.new(width=14, height=7)
par(mfrow=c(1, 2))
boxplot(rt(100, 5), xlab=NA, notch=TRUE, main="Boxplot")
plot(rnorm(10), pch=16, xlab=NA, ylab=NA, main="Streudiagramm")
